@@ -26,6 +26,7 @@
 #include "wt.h"
 #include "framebuf.h"
 #include "graphics.h"
+#include "input.h"
 #include "error.h"
 
 
@@ -34,6 +35,7 @@ void fatal_error(char *message, ...)
      va_list args;
 
      end_graphics();
+     end_input_devices();
 
      va_start(args, message);
      vfprintf(stderr, message, args);
