@@ -28,8 +28,8 @@
 /* fixed point conversions */
 #define INT_TO_FIXED(i) ((i) << 16)
 #define FIXED_TO_INT(f) ((f) >> 16)
-#define FIXED_TO_FLOAT(f) (((double) (f)) / 65536.0)
-#define FLOAT_TO_FIXED(f) ((fixed) (f * 65536.0))
+#define FIXED_TO_FLOAT(f) (((double) (f)) * 1.52587890625e-5)
+#define FLOAT_TO_FIXED(f) ((fixed) ((f) * 65536.0))
 
 /* functions */
 #define FIXED_ABS(f) ((f) < 0 ? -(f) : (f))
@@ -106,13 +106,67 @@ static inline fixed fixdiv(fixed dividend, fixed divisor)
      return result;
 }
 
+#elif defined(ARCH_MAC) && !defined(__powerc)
+
+     /* MBW - 68k Macintosh only... */
+
+     /* toolbox trap versions - slow... */
+     /* extern pascal fixed fixmul(fixed a, fixed b) = 0xA868; */
+     /* extern pascal fixed fixmul2_30(fixed x, fixed y) = 0xA84A; */
+
+     /* inline versions */
+     #pragma parameter __D0 fixmul(__D0,__D1)
+     fixed fixmul(fixed a, fixed b) = 
+     {
+             0x4C01,0x0C01   // muls.L D1,D1:D0
+             ,0x3001                 // move.w D1,D0
+             ,0x4840                 // swap D0
+     };
+     #pragma parameter __D1 fixmul2_30(__D0,__D1)
+     fixed fixmul2_30(fixed a, fixed b) = 
+     {
+             0x4C01,0x0C01   // muls.L D1,D1:D0
+             ,0xE589                 // lsl.L #2, D1
+             /* without the following lines, we're dropping the 
+                least signifigant 2 bits.  If this is a problem, 
+                uncomment them. 
+              */
+              //,0xE598                       // rol.L #2, D0
+              //,0xEFC1, 0x0782       // bfins D0, D1{30,2}
+     };
+
+     #undef INT_TO_FIXED
+     #undef FIXED_TO_INT
+     #undef FIXED_TRUNC
+
+     #pragma parameter __D0 FIXED_TRUNC(__D0)
+     fixed FIXED_TRUNC(fixed x) = 
+             0x4240;         // clr.W D0
+
+     #pragma parameter __D0 FIXED_TO_INT(__D0)
+     signed short FIXED_TO_INT(fixed x) = 
+             0x4840;         // swap D0
+
+     #pragma parameter __D0 INT_TO_FIXED(__D0)
+     fixed INT_TO_FIXED(signed short x) = 
+     {
+             0x4840          // swap D0
+             ,0x4240         // clr.w D0
+     };
+
+     /* MBW - one of these days I'll rewrite division too. 
+        The toolbox trap works, though.
+     */
+     extern pascal fixed fixdiv(fixed x, fixed y) = 0xA84D;
+
 #else
+        #define fixmul(a, b) ((fixed) ((double) (a) * ((double) (b)) * \
+				       1.52587890625e-5))
+        #define fixmul2_30(a, b) ((fixed) ((double) (a) * ((double) (b) * \
+					    9.313225746154785e-10)))
+        #define fixdiv(a, b) ((fixed) (((double) (a) / (double) (b)) * 65536.0))
 
-#define fixmul(a, b) ((fixed) (((double) (a) * (double) (b)) / 65536.0))
-#define fixmul2_30(a, b) ((fixed) (((double) (a) * (double) (b)) / \
-				   1073741824.0))
-#define fixdiv(a, b) ((fixed) (((double) (a) / (double) (b)) * 65536.0))
+#endif /* ARCH_* */
 
-#endif
 
 #endif /* _FIXED_H_ */
