@@ -20,7 +20,12 @@
 **  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-typedef unsigned char Pixel;
+
+#ifdef TRUECOLOR
+typedef int16 Pixel;
+#else
+typedef int8 Pixel;
+#endif
 
 typedef struct {
      int fb_width, fb_height;
@@ -30,5 +35,3 @@ typedef struct {
 
 extern Framebuffer *new_framebuffer(int width, int height);
 extern void clear_framebuffer(Framebuffer *fb);
-
-
