@@ -20,13 +20,21 @@
 **  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#define SCREEN_WIDTH  320
-#define SCREEN_HEIGHT 200
 
-#define PALETTE_ENTRIES 192
+typedef struct {
+     int rgb_cube_size;
+     long *color_lookup;
+} Palette_info;
+
+typedef struct {
+     int width, height;
+     Palette_info palette;
+} Graphics_info;
+
+typedef int8 DevicePixel;
 
 
-extern void init_graphics(void);
+extern Graphics_info *init_graphics(int width, int height, Boolean blocky);
 extern void end_graphics(void);
 extern void update_screen(Framebuffer *fb);
 extern Pixel *get_framebuffer_memory(int width, int height);
