@@ -21,12 +21,20 @@
 */
 
 
-#define INTENT_END_GAME 1
-#define INTENT_JUMP     2
+#define INTENT_END_GAME 0
+#define INTENT_JUMP     1
+#define INTENT_ACTION1  2
+#define INTENT_ACTION2  3
+#define INTENT_ACTION3  4
+#define INTENT_ACTION4  5
+#define INTENT_ACTION5  6
 
 
 #define MAX_SPECIAL_INTENTIONS  20
 
+#define MOVE_FORCE 30.0
+#define TURN_FORCE  5.0
+
 typedef struct {
      double force_x, force_y, force_z;
      double force_rotate;
