@@ -59,6 +59,8 @@ Intent *read_input_devices(void)
      intent.n_special = 0;
 
      keyboard_update();
+
+     /* Movement keys. */
      if (keyboard_keypressed(SCANCODE_CURSORBLOCKLEFT))
 	  rotating_ccw = True;
      if (keyboard_keypressed(SCANCODE_CURSORBLOCKRIGHT))
@@ -69,14 +71,26 @@ Intent *read_input_devices(void)
 	  moving_backward= True;
      if (keyboard_keypressed(42) || keyboard_keypressed(54))
 	  running= True;
-     if (keyboard_keypressed(57))
-	  add_special(&intent, INTENT_JUMP);
      if (keyboard_keypressed(56) || keyboard_keypressed(100))
 	  strafing= True;
 
+     /* Specials. */
+     if (keyboard_keypressed(57))
+	  add_special(&intent, INTENT_JUMP);
+     if (keyboard_keypressed(SCANCODE_F1))
+	  add_special(&intent, INTENT_ACTION1);
+     if (keyboard_keypressed(SCANCODE_F2))
+	  add_special(&intent, INTENT_ACTION2);
+     if (keyboard_keypressed(SCANCODE_F3))
+	  add_special(&intent, INTENT_ACTION3);
+     if (keyboard_keypressed(SCANCODE_F4))
+	  add_special(&intent, INTENT_ACTION4);
+     if (keyboard_keypressed(SCANCODE_F5))
+	  add_special(&intent, INTENT_ACTION5);
+
      if (!keyboard_keypressed(SCANCODE_CURSORBLOCKLEFT))
 	  rotating_ccw = False;
-     if (!keyboard_keypressed( SCANCODE_CURSORBLOCKRIGHT))
+     if (!keyboard_keypressed(SCANCODE_CURSORBLOCKRIGHT))
 	  rotating_cw = False;
      if (!keyboard_keypressed(SCANCODE_CURSORBLOCKUP))
 	  moving_forward = False;
@@ -91,20 +105,20 @@ Intent *read_input_devices(void)
 
      if (rotating_cw) {
 	  if (strafing)
-	       intent.force_y -= 0.5;
+	       intent.force_y -= MOVE_FORCE;
 	  else
-	       intent.force_rotate -= 0.5;
+	       intent.force_rotate -= TURN_FORCE;
      }
      if (rotating_ccw) {
 	  if (strafing) {
-	       intent.force_y += 0.5;
+	       intent.force_y += MOVE_FORCE;
 	  } else
-	       intent.force_rotate += 0.5;
+	       intent.force_rotate += TURN_FORCE;
      }
      if (moving_forward)
-	  intent.force_x += 0.5;
+	  intent.force_x += MOVE_FORCE;
      if (moving_backward)
-	  intent.force_x -= 0.5;
+	  intent.force_x -= MOVE_FORCE;
      if (running) {
 	  intent.force_x *= 2.0;
 	  intent.force_y *= 2.0;
