@@ -20,26 +20,18 @@
 **  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#define PROFILE 1
 
 #include <math.h>
 #include <stdlib.h>
 #include <string.h>
-#if PROFILE
-#include <sys/types.h>
-#include <unistd.h>
-#include <sys/time.h>
-#include <stdio.h>
-#endif
 #include "wt.h"
 #include "error.h"
 #include "fixed.h"
 #include "wtmem.h"
 #include "table.h"
 #include "view.h"
-#include "texture.h"
 #include "framebuf.h"
-#include "graphics.h"
+#include "texture.h"
 #include "world.h"
 #include "render.h"
 
@@ -57,7 +49,7 @@
 #define FROM_FIX_8_24(f) ((f) >> 8) 
 #define MAX_WALL_EVENTS 30
 
-#define MAX_ERROR  SCREEN_WIDTH
+#define MAX_ERROR  VIEW_WIDTH
 
 
 typedef struct {
@@ -135,13 +127,15 @@ static int *fb_rows = NULL;
 static View_constants view_constants;
 
 
-#if defined(__GNUC__) && defined(ARCH_i86)
+#if defined(__GNUC__) && defined(ARCH_i86) && !defined(DEBUG)
+/* It would be a Bad Thing to use the Intel assembly functions while debugging,
+**   as they clobber the frame pointer.
+*/
 #include "slice-gas86.c"
 #else
 #include "slice.c"
 #endif
 
-
 void init_renderer(int fb_width, int fb_height)
 {
      int i;
@@ -165,71 +159,16 @@ void init_renderer(int fb_width, int fb_height)
 }
 
 
-#if PROFILE
-static double current_time(void)
+Framebuffer *render(World *w, View *v)
 {
-     struct timeval tv;
-     struct timezone tz;
-
-     tz.tz_minuteswest = 0;
-     tz.tz_dsttime = DST_NONE;
-     gettimeofday(&tv, &tz);
-
-     return (double) tv.tv_sec + (double) tv.tv_usec / 1000000.0;
-}
-#endif
-
-#ifdef ARCH_SUN
-static void *memmove(void *dest, void *src, size_t size)
-{
-     int *lsrc = (int *) src;
-     int *ldest = (int *) dest;
-
-     size /= 4;
-     /* Here, we assume that size is a multiple of four.  That's the only
-     **   way in which memmove will be used.  This assumption is nasty,
-     **   but life is rough . . . I'm bitter about having to implement
-     **   memmove in the first place.
-     */
-     if (lsrc < ldest) {
-	  ldest += size - 1;
-	  lsrc += size - 1;
-	  while (size-- > 0)
-	       *ldest-- = *lsrc--;
-     } else {
-	  while (size-- > 0)
-	       *ldest++ = *lsrc++;
-     }
-
-     return dest;
-}
-#endif
-
-
-void render(World *w, View *v)
-{
-#if PROFILE
-     static int fps_count = 0;
-     static double total_time = 0.0;
-     double start_time = current_time();
-#endif
      init_buffers();
      calc_view_constants(v, VIEW_WIDTH, VIEW_HEIGHT);
      transform_vertices(w, v);
      clip_walls(w, v);
      clear_framebuffer(fb);
      render_walls(w, v);
-     update_screen(fb);
 
-#if PROFILE
-     fps_count++;
-     total_time += current_time() - start_time;
-     if (fps_count == 100) {
-	  printf("fps = %3.2f\n", (double) fps_count / total_time);
-	  fps_count = 0;
-	  total_time = 0.0;
-     }
-#endif
+     return fb;
 }
 
 
@@ -312,21 +251,21 @@ static void clip_walls(World *w, View *v)
 	  /* check for trivial accept */
 	  if ((outcode1 | outcode2) != 0) {
 	       /* Damn . . . we need to clip. */
-	       fixed slope, denom, y_diff;
+	       fixed base_slope, slope, denom, y_diff;
 
 	       denom = (x2 - x1);
 	       if (FIXED_ABS(denom) < FIXED_EPSILON) {
 		    if (denom < 0)
-			 slope = FIXED_MIN + v->view_plane_size;
+			 base_slope = FIXED_MIN + v->view_plane_size;
 		    else
-			 slope = FIXED_MAX - v->view_plane_size;
+			 base_slope = FIXED_MAX - v->view_plane_size;
 	       } else
-		    slope = fixdiv(y2 - y1, denom);
+		    base_slope = fixdiv(y2 - y1, denom);
 
 	       if (outcode1 == 1) {
 		    px1 = -v->view_plane_size;
 		    y_diff = y1 - fixmul(x1, -v->view_plane_size);
-		    slope += v->view_plane_size;
+		    slope = base_slope + v->view_plane_size;
 		    if (FIXED_ABS(slope) > FIXED_EPSILON)
 			 x1 -= fixdiv(y_diff, slope);
 		    else
@@ -334,7 +273,7 @@ static void clip_walls(World *w, View *v)
 	       } else if (outcode1 == 2) {
 		    px1 = v->view_plane_size;
 		    y_diff = y1 - fixmul(x1, v->view_plane_size);
-		    slope -= v->view_plane_size;
+		    slope = base_slope - v->view_plane_size;
 		    if (FIXED_ABS(slope) > FIXED_EPSILON)
 			 x1 -= fixdiv(y_diff, slope);
 		    else
@@ -344,7 +283,7 @@ static void clip_walls(World *w, View *v)
 	       if (outcode2 == 1) {
 		    px2 = -v->view_plane_size;
 		    y_diff = y2 - fixmul(x2, -v->view_plane_size);
-		    slope += v->view_plane_size;
+		    slope = base_slope + v->view_plane_size;
 		    if (FIXED_ABS(slope) > FIXED_EPSILON)
 			 x2 -= fixdiv(y_diff, slope);
 		    else
@@ -352,7 +291,7 @@ static void clip_walls(World *w, View *v)
 	       } else if (outcode2 == 2) {
 		    px2 = v->view_plane_size;
 		    y_diff = y2 - fixmul(x2, v->view_plane_size);
-		    slope -= v->view_plane_size;
+		    slope = base_slope - v->view_plane_size;
 		    if (FIXED_ABS(slope) > FIXED_EPSILON)
 			 x2 -= fixdiv(y_diff, slope);
 		    else
@@ -446,7 +385,8 @@ static void render_walls(World *w, View *v)
 	  if (active == NULL)
 	       active = wtmalloc(sizeof(Active_wall) * last_wall_count);
 	  else
-	       active = wtrealloc(active, sizeof(int) * last_wall_count);
+	       active = wtrealloc(active,
+				  sizeof(Active_wall) * last_wall_count);
 
 	  /* I know this is really cheesy, but . . . */
 	  if (intersections == NULL)
@@ -733,7 +673,7 @@ static void draw_walls(Wall_intersection *int_list, View *v)
      int i = 0;
      int fb_column = VIEW_WIDTH - 1;
      unsigned int tex_column, tex_mask;
-     unsigned char *tex_base;
+     Pixel *tex_base;
      fixed tex_y, tex_dy;
      Pixel *fb_byte, *last_byte;
      fixed pstart1, pend1, pstart2, pend2;
@@ -837,7 +777,7 @@ static void draw_walls(Wall_intersection *int_list, View *v)
 		    tex_dy = fixdiv(wall->yscale, FIXED_SCALE(z, VIEW_HEIGHT));
 	  }
 	  tex_mask = texture->height - 1;
-	  tex_base = texture->texels + (tex_column << texture->log2width);
+	  tex_base = texture->texels + (tex_column << texture->log2height);
 
 	  /* Now we can actually draw the walls, starting with
 	  **   the floor segment.
@@ -849,22 +789,19 @@ static void draw_walls(Wall_intersection *int_list, View *v)
 		    pstart1 = int_list->bottom;
 		    start1 = fixdiv(pstart1, z) + height;
 	       }
-	       if (pend1 > int_list->top)
-		    pend1 = int_list->top;
+	       if (pend1 >= int_list->top)
+		    pend1 = int_list->top - 1;
 
 	       fb_start = FIXED_TO_INT(FIXED_SCALE(pstart1, VIEW_HEIGHT));
 	       fb_end = FIXED_TO_INT(FIXED_SCALE(pend1, VIEW_HEIGHT));
+	       fb_start = (VIEW_HEIGHT >> 1) - 1 - fb_start;
+	       fb_end = (VIEW_HEIGHT >> 1) - 1 - fb_end;
 
-	       fb_start = (VIEW_HEIGHT >> 1) - fb_start;
-	       /* The following line of code is a hack . . . */
-	       if (fb_start >= VIEW_HEIGHT)
-		    fb_start = VIEW_HEIGHT - 1;
-	       fb_end = (VIEW_HEIGHT >> 1) - fb_end;
 	       fb_byte = fb->pixels + fb_column + fb_rows[fb_start];
 	       last_byte = fb->pixels + fb_column + fb_rows[fb_end];
 	       if (wall->sky)
 		    tex_y = fixmul(view_constants.row_view[fb_start],
-				   wall->yscale);
+				   wall->yscale) + wall->yphase;
 	       else
 		    tex_y = fixmul(start1, wall->yscale) + wall->yphase;
 
@@ -881,21 +818,19 @@ static void draw_walls(Wall_intersection *int_list, View *v)
 		    pstart2 = int_list->bottom;
 		    start2 = fixdiv(pstart2, z) + height;
 	       }
-	       if (pend2 > int_list->top)
-		    pend2 = int_list->top;
+	       if (pend2 >= int_list->top)
+		    pend2 = int_list->top - 1;
 
 	       fb_start = FIXED_TO_INT(FIXED_SCALE(pstart2, VIEW_HEIGHT));
 	       fb_end = FIXED_TO_INT(FIXED_SCALE(pend2, VIEW_HEIGHT));
+	       fb_start = (VIEW_HEIGHT >> 1) - 1 - fb_start;
+	       fb_end = (VIEW_HEIGHT >> 1) - 1 - fb_end;
 
-	       fb_start = (VIEW_HEIGHT >> 1) - fb_start;
-	       /* The following line of code is a hack . . . */
-	       if (fb_start >= VIEW_HEIGHT)
-		    fb_start = VIEW_HEIGHT - 1;
-	       fb_end = (VIEW_HEIGHT >> 1) - fb_end;
 	       fb_byte = fb->pixels + fb_column + fb_rows[fb_start];
 	       last_byte = fb->pixels + fb_column + fb_rows[fb_end];
 	       if (wall->sky)
-		    tex_y = view_constants.row_view[fb_start];
+		    tex_y = fixmul(view_constants.row_view[fb_start],
+				   wall->yscale) + wall->yphase;
 	       else
 		    tex_y = fixmul(start2, wall->yscale) + wall->yphase;
 
@@ -984,6 +919,9 @@ static void draw_floors(Wall_intersection *int_list, View *v)
 	  /* draw ceiling slices generated by floor walls */
 	  while (last->wall != NULL) {
 	       start = MAX(last_max_floor, last->pend2);
+/*
+	       start = last->pend2;
+*/
 	       end = MIN(max_floor, last->top);
 	       if (start < end)
 		    draw_ceiling_slices(last->front, start, end, v, i - 1);
@@ -1035,14 +973,14 @@ static void draw_floor_slices(Region *r, fixed start, fixed end,
      fixed sin_x, cos_x;
 
 
-     start_row = FIXED_TO_INT(start * VIEW_HEIGHT);
-     end_row = FIXED_TO_INT(end * VIEW_HEIGHT);
+     start_row = FIXED_TO_INT(FIXED_SCALE(start, VIEW_HEIGHT));
+     end_row = FIXED_TO_INT(FIXED_SCALE(end, VIEW_HEIGHT));
      /* Bail out early and save time if there's nothing to draw. */
      if (start_row == end_row)
 	  return;
 
-     start_row = (VIEW_HEIGHT >> 1) - start_row;
-     end_row = (VIEW_HEIGHT >> 1) - end_row;
+     start_row = (VIEW_HEIGHT >> 1) - 1 - start_row;
+     end_row = (VIEW_HEIGHT >> 1) - 1 - end_row;
      if (start_row >= VIEW_HEIGHT)
 	  start_row = VIEW_HEIGHT - 1;
      if (end_row < 0)
@@ -1086,14 +1024,14 @@ static void draw_ceiling_slices(Region *r, fixed start, fixed end,
      fixed sin_x, cos_x;
 
 
-     start_row = FIXED_TO_INT(start * VIEW_HEIGHT);
-     end_row = FIXED_TO_INT(end * VIEW_HEIGHT);
+     start_row = FIXED_TO_INT(FIXED_SCALE(start, VIEW_HEIGHT));
+     end_row = FIXED_TO_INT(FIXED_SCALE(end, VIEW_HEIGHT));
      /* Bail out early and save time if there's nothing to draw. */
      if (start_row == end_row)
 	  return;
 
-     start_row = (VIEW_HEIGHT >> 1) - start_row;
-     end_row = (VIEW_HEIGHT >> 1) - end_row;
+     start_row = (VIEW_HEIGHT >> 1) - 1 - start_row;
+     end_row = (VIEW_HEIGHT >> 1) - 1 - end_row;
      if (start_row >= VIEW_HEIGHT)
 	  start_row = VIEW_HEIGHT - 1;
      if (end_row < 0)
