@@ -21,4 +21,4 @@
 */
 
 extern void init_renderer(int fb_width, int fb_height);
-extern void render(World *w, View *v);
+extern Framebuffer *render(World *w, View *v);
