@@ -21,42 +21,100 @@
 */
 
 
-inline void draw_wall_slice(Pixel *start, Pixel *end, unsigned char *tex_base,
+#define PIXEL_GUARD_VAL ((Pixel) ~0)
+
+
+inline void draw_wall_slice(Pixel *start, Pixel *end, Pixel *tex_base,
+			    fixed tex_y, fixed tex_dy, int fb_width, 
+			    int tex_height, int npix);
+inline void draw_floor_slice(Pixel *start, Pixel *tex,
+			     fixed x, fixed y, fixed dx, fixed dy,
+			     int tex_width);
+
+
+
+inline void draw_wall_slice(Pixel *start, Pixel *end, Pixel *tex_base,
 			    fixed tex_y, fixed tex_dy, int fb_width, 
 			    int tex_height, int npix)
 {
-     unsigned mask = tex_height - 1;
+     /* npix is not used in this version of draw_wall_slice, but it is
+     **   used in others for loop unrolling.
+     */
+
+     if (tex_height == 64) {
+
+	  tex_y <<= 10;
+	  tex_dy <<= 10;
+
+	  while (start >= end) {
+	       *start = tex_base[tex_y >> 26];
+	       tex_y += tex_dy;
+	       start -= fb_width;
+	  }
+
+     } else {
+
+	  tex_y <<= 9;
+	  tex_dy <<= 9;
+
+	  while (start >= end) {
+	       *start = tex_base[tex_y >> 25];
+	       tex_y += tex_dy;
+	       start -= fb_width;
+	  }
 
-     while (start >= end) {
-	  *start = tex_base[FIXED_TO_INT(tex_y) & mask];
-	  tex_y += tex_dy;
-	  start -= fb_width;
      }
 }
 
 
-inline void draw_floor_slice(Pixel *start, unsigned char *tex,
+inline void draw_floor_slice(Pixel *start, Pixel *tex,
 			     fixed x, fixed y, fixed dx, fixed dy,
 			     int tex_width)
 {
-     if (*start != 255)
+     register Pixel temp;
+
+
+     temp = *start;
+     if (temp != PIXEL_GUARD_VAL)
 	  return;
 
      if (tex_width == 64) {
-	  while (*start == 255) {
+	  /* width of 64 == 2^6 */
+	  y <<= 6;
+	  dy <<= 6;
+
+	  do {
+
 	       x &= 0x3fffff;
-	       y &= 0x3fffff;
-	       *start++ = tex[((y >> 16) << 6) + (x >> 16)];
+	       /* No need to mask y--it rolls automatically *if* ints
+               **   are 32 bits.
+	       */
+
+	       temp = tex[FIXED_TO_INT((y & 0xfc00000) | x)];
+	       *start++ = temp;
 	       x += dx;
+	       temp = *start;
 	       y += dy;
-	  }
+
+	  } while (temp == PIXEL_GUARD_VAL);
+
      } else {
-	  while (*start == 255) {
+
+	  /* tex_width == 128 == 2^7*/
+	  y <<= 7;
+	  dy <<= 7;
+
+	  do {
+
 	       x &= 0x7fffff;
-	       y &= 0x7fffff;
-	       *start++ = tex[((y >> 16) << 7) + (x >> 16)];
+
+	       temp = tex[FIXED_TO_INT((y & 0x3f800000) | x)];
+	       *start++ = temp;
 	       x += dx;
+	       temp = *start;
 	       y += dy;
-	  }
+
+	  } while (temp == PIXEL_GUARD_VAL);
+
      }
 }
