@@ -21,6 +21,7 @@
 */
 
 typedef struct {
+     Boolean fixed_size;
      size_t entry_size;
      int current_entries;
      int max_entries;
@@ -31,7 +32,6 @@ typedef struct {
 #define TABLE_ELEMENTS(t, type)   ((type *) (t)->table)
 #define TABLE_SIZE(t)             ((t)->current_entries)
 
-extern Table *new_table(size_t entry_size);
-extern void init_table(Table *t, size_t entry_size);
-extern void add_table_entry(Table *t, void *entry);
+extern Table *new_table(size_t entry_size, int table_size);
+extern int add_table_entry(Table *t, void *entry);
 extern int table_size(Table *t);
