@@ -25,14 +25,25 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <assert.h>
 #include "wt.h"
 #include "wtmem.h"
 #include "error.h"
+#include "color.h"
+#include "graphfile.h"
+#include "framebuf.h"
+#include "graphics.h"
 #include "texture.h"
 
 
-static TexFormat check_texture_file(FILE *fp, char *filename);
-static int log2(int x);
+#define COLOR_LEVELS 6
+
+
+static int ilog2(int x);
+static void quantize_texture(Graphic_file *gfile, Texture *t);
+
+static int tex_rgb_cube_size;
+static long *tex_color_lookup = NULL;
 
 
 Texture *new_texture(int width, int height)
@@ -43,11 +54,11 @@ Texture *new_texture(int width, int height)
 	  fatal_error("new_texture:  bad texture dimensions");
 
      t = wtmalloc(sizeof(Texture));
-     t->texels = wtmalloc(height * width);
+     t->texels = wtmalloc(height * width * sizeof(Pixel));
 
      t->width = width;
      t->height = height;
-     t->log2width = log2(width);
+     t->log2height = ilog2(height);
      
      return t;
 }
@@ -55,71 +66,46 @@ Texture *new_texture(int width, int height)
 
 Texture *read_texture_file(char *filename)
 {
-     FILE *fp;
+     Graphic_file *gfile;
      Texture *t;
-     int height, width;
 
 
-     if (!(fp = fopen(filename, "rb")))
-	  fatal_error("Cannot open texture file %s.", filename);
-
-     switch (check_texture_file(fp, filename)) {
-	case WTTEX:
-	  break;
-
-	case GIF87a:
-	  return LoadGIF(fp, filename);
-	  break;
-
-	default:
-	  fatal_error("%s is not a texture file (bad magic number).",
-		      filename);
-	  break;
-     }
-
-     if (fscanf(fp, "%d %d\n", &width, &height) != 2)
-	  fatal_error("Texture file %s has a bad header.", filename);
+     gfile = read_graphic_file(filename);
+     if (gfile == NULL)
+	  fatal_error("Error reading texture %s.", filename);
 
      /* The height and width should be powers of two for efficient
      **   texture mapping.  Here, we enforce this.
      */
-     if (log2(width) == -1 || log2(height) == -1)
+     if (ilog2(gfile->width) == -1 || ilog2(gfile->height) == -1)
 	  fatal_error("Dimensions texture %s are not powers of two.", 
 		      filename);
-     t = new_texture(width, height);
+     
+     t = new_texture(gfile->width, gfile->height);
+     quantize_texture(gfile, t);
 
-     if (fread(t->texels, 1, t->height * t->width, fp) !=
-	 t->height * t->width)
-	  fatal_error("Error reading texture file %s.", filename);
-
-     fclose(fp);
+     free_graphic_file(gfile);
 
      return t;
 }
 
 
-/* in what format are the texturefile */
-static TexFormat check_texture_file( FILE *fp, char *filename )
+/* Set texture translation parameters.  This is sort of kludgy--the rest
+**   the world loading code does not need any information about the graphics
+**   device.  But here, we need to convert the texture maps to a one byte
+**   per pixel format using the color map of output device.
+*/
+void set_texture_trans(int rgb_cube_size, long *color_lookup)
 {
-     char magic[TEXTURE_MAGIC_LENGTH];
-
-     if (fread(magic, 1, TEXTURE_MAGIC_LENGTH, fp) != TEXTURE_MAGIC_LENGTH)
-	  fatal_error("Error reading texture file %s.", filename);
-
-     if (strncmp(magic, TEXTURE_MAGIC, TEXTURE_MAGIC_LENGTH) == 0)
-	  return WTTEX;
-
-     if (strncmp(magic, GIF_MAGIC, TEXTURE_MAGIC_LENGTH) == 0)
-	  return GIF87a;
-
-     return Unknown;
+     tex_rgb_cube_size = rgb_cube_size;
+     tex_color_lookup = color_lookup;
 }
 
 
 /* Return the log base 2 of the argument, or -1 if the argument is not
 **   an integer power of 2.
 */
-static int log2(int x)
+static int ilog2(int x)
 {
      int i;
      unsigned int n;
@@ -128,7 +114,6 @@ static int log2(int x)
 	  return -1;
      else
 	  n = (unsigned int) x;
-
      
      for (i = 0; (n & 0x1) == 0; i++, n >>= 1);
 
@@ -137,3 +122,32 @@ static int log2(int x)
      else
 	  return -1;
 }
+
+
+static void quantize_texture(Graphic_file *gfile, Texture *t)
+{
+     RGBcolor pixel;
+     int x, y;
+     int color_divisor = 256 / tex_rgb_cube_size;
+     Pixel *dest_pixel = t->texels;
+
+
+     for (x = gfile->width - 1; x >= 0; x--)
+     for (y = gfile->height - 1; y >= 0; y--) {
+	  int red, green, blue;
+
+	  pixel = graphic_file_pixel(gfile, x, y);
+
+	  red = MIN(pixel.red / color_divisor, tex_rgb_cube_size - 1);
+	  green = MIN(pixel.green / color_divisor, tex_rgb_cube_size - 1);
+	  blue = MIN(pixel.blue / color_divisor, tex_rgb_cube_size - 1);
+
+#ifndef TRUECOLOR
+	  *dest_pixel++ = tex_color_lookup[(red * tex_rgb_cube_size + green) *
+					   tex_rgb_cube_size + blue];
+#else
+	  *dest_pixel++ = (red * tex_rgb_cube_size + green) *
+ 	                   tex_rgb_cube_size + blue;
+#endif
+     }
+}
