@@ -25,24 +25,15 @@
 #ifndef TEXTURE_H_
 #define TEXTURE_H_
 
-#define TEXTURE_MAGIC "wttex"
-#define TEXTURE_MAGIC_LENGTH 5
-
-#define GIF_MAGIC "GIF87"
-
 
 typedef struct {
      int width, height;
-     int log2width;
-     unsigned char *texels;
+     int log2height;
+     Pixel *texels;
 } Texture;
 
-typedef enum { WTTEX, GIF87a, Unknown } TexFormat;
-
 extern Texture *new_texture(int width, int height);
 extern Texture *read_texture_file(char *filename);
-
-/* The following function is located in gifload.c, not texture.c. */
-extern Texture *LoadGIF(FILE *fp, char *fname);
+extern void set_texture_trans(int cube_size, long *color_lookup);
 
 #endif
