@@ -27,51 +27,47 @@
 #include "error.h"
 #include "fixed.h"
 #include "wtmem.h"
+#include "framebuf.h"
 #include "texture.h"
 #include "table.h"
 #include "world.h"
 
 
-#define START_VERTICES 50
-#define START_WALLS    30
-#define START_TEXTURES 10
-
-
 World *new_world(void)
 {
      World *w;
 
      w = wtmalloc(sizeof(World));
-     w->vertices = new_table(sizeof(Vertex));
-     w->walls = new_table(sizeof(Wall));
-     w->regions = new_table(sizeof(Region));
-     w->textures = new_table(sizeof(Texture *));
+     w->vertices = new_table(sizeof(Vertex), MAX_VERTICES);
+     w->walls = new_table(sizeof(Wall), MAX_WALLS);
+     w->regions = new_table(sizeof(Region), MAX_REGIONS);
+     w->textures = new_table(sizeof(Texture *), MAX_TEXTURES);
 
      return w;
 }
 
 
-void add_texture(World *w, Texture *tex)
+int add_texture(World *w, Texture *tex)
 {
-     add_table_entry(w->textures, &tex);
+     return add_table_entry(w->textures, &tex);
 }
 
 
-void add_wall(World *w, Wall *wall)
+int add_wall(World *w, Wall *wall)
 {
-     add_table_entry(w->walls, wall);
+     return add_table_entry(w->walls, wall);
 }
 
 
-void add_vertex(World *w, Vertex *v)
+int add_vertex(World *w, Vertex *v)
 {
-     add_table_entry(w->vertices, v);
+     return add_table_entry(w->vertices, v);
 }
 
 
-void add_region(World *w, Region *r)
+int add_region(World *w, Region *r)
 {
-     add_table_entry(w->regions, r);
+     return add_table_entry(w->regions, r);
 }
 
 
