@@ -22,6 +22,11 @@
 
 #define NO_WALL -1
 
+#define MAX_VERTICES 2000
+#define MAX_WALLS    2000
+#define MAX_REGIONS  2000
+#define MAX_TEXTURES  200
+
 typedef struct {
      fixed x, y;
      fixed tx, ty, proj;      /* transformed coordinates */
@@ -59,10 +64,10 @@ typedef struct {
 #define WORLD_WALL(w, x)     ((TABLE_ELEMENTS((w)->walls, Wall))[x])
 
 extern World *new_world(void);
-extern void add_texture(World *w, Texture *tex);
-extern void add_wall(World *w, Wall *wall);
-extern void add_vertex(World *w, Vertex *v);
-extern void add_region(World *w, Region *r);
+extern int add_texture(World *w, Texture *tex);
+extern int add_wall(World *w, Wall *wall);
+extern int add_vertex(World *w, Vertex *v);
+extern int add_region(World *w, Region *r);
 extern void update_wall_scale(World *w, int wall_num,
 			      fixed xscale, fixed yscale);
 extern void update_wall_phase(World *w, int wall_num,
