@@ -33,6 +33,7 @@
 #include "wtmem.h"
 #include "list.h"
 #include "table.h"
+#include "framebuf.h"
 #include "texture.h"
 #include "world.h"
 #include "worldfile.h"
@@ -42,7 +43,7 @@
 #define STRING_TOKEN_MAX_LENGTH 256
 
 #define IS_STRING_TOKEN_CHAR(c) (isalpha(c) || isdigit(c) || \
-				 (c) == '/' || c == '.')
+				 (c) == '/' || c == '.' || c == '_')
 #define COMMENT_CHAR ';'
 
 typedef struct {
@@ -63,6 +64,7 @@ static void parse_vertex(FILE *fp, World *w);
 static void parse_wall(FILE *fp, World *w);
 static void parse_region(FILE *fp, World *w);
 static void parse_texture(FILE *fp, World *w);
+static void parse_start(FILE *fp);
 static int get_texture_index(char *name);
 static List_result find_list_name(void *node, void *data);
 static Token_type get_string_token(FILE *fp, char *tokenbuf);
@@ -101,6 +103,8 @@ void parse_world(FILE *fp, World *w)
 	       parse_region(fp, w);
 	  else if (strcmp(tokenbuf, "vertex") == 0)
 	       parse_vertex(fp, w);
+	  else if (strcmp(tokenbuf, "start") == 0)
+	       parse_start(fp);
 	  else
 	       parse_error("unknown structure type");
      }
@@ -232,6 +236,8 @@ void parse_wall(FILE *fp, World *w)
      wall.xscale = fixmul(fixmul(wall.xscale,
 				 INT_TO_FIXED(wall.surface_texture->width)),
 			  wall_length);
+     wall.xphase = FIXED_SCALE(wall.xphase, wall.surface_texture->width);
+     wall.yphase = FIXED_SCALE(wall.yphase, wall.surface_texture->height);
 
      add_wall(w, &wall);
 }
@@ -266,13 +272,28 @@ void parse_texture(FILE *fp, World *w)
 }
 		 
 
+void parse_start(FILE *fp)
+{
+     extern fixed start_x, start_y;
+     fixed x, y;
+
+     if (get_real_token(fp, &x) != Token_real)
+	  parse_error("number expected");
+     if (get_real_token(fp, &y) != Token_real)
+	  parse_error("number expected");
+
+     start_x = x;
+     start_y = y;
+}
+
+
 int get_texture_index(char *name)
 {
      List *l;
 
      l = scan_list(texture_list, name, find_list_name);
-     if (l == NULL)
-	  return -1;
+     if (l == NULL || l->next == NULL || l->next->node == NULL)
+          fatal_error("get_texture_index:  could not find %s\n", name);
      else
 	  return ((Texture_node *) l->next->node)->index;
 }
