@@ -113,6 +113,26 @@ Intent *read_input_devices(void)
 		    add_special(&intent, INTENT_END_GAME);
 		    break;
 
+		  case XK_1:
+		    add_special(&intent, INTENT_ACTION1);
+		    break;
+
+		  case XK_2:
+		    add_special(&intent, INTENT_ACTION2);
+		    break;
+
+		  case XK_3:
+		    add_special(&intent, INTENT_ACTION3);
+		    break;
+
+		  case XK_4:
+		    add_special(&intent, INTENT_ACTION4);
+		    break;
+
+		  case XK_5:
+		    add_special(&intent, INTENT_ACTION5);
+		    break;
+
 		  default:
 		    break;
 	       }
@@ -160,20 +180,20 @@ Intent *read_input_devices(void)
      
      if (rotating_cw) {
 	  if (strafing)
-	       intent.force_y -= 0.5;
+	       intent.force_y -= MOVE_FORCE;
 	  else
-	       intent.force_rotate -= 0.5;
+	       intent.force_rotate -= TURN_FORCE;
      }
      if (rotating_ccw) {
 	  if (strafing) {
-	       intent.force_y += 0.5;
+	       intent.force_y += MOVE_FORCE;
 	  } else
-	       intent.force_rotate += 0.5;
+	       intent.force_rotate += TURN_FORCE;
      }
      if (moving_forward)
-	  intent.force_x += 0.5;
+	  intent.force_x += MOVE_FORCE;
      if (moving_backward)
-	  intent.force_x -= 0.5;
+	  intent.force_x -= MOVE_FORCE;
      if (running) {
 	  intent.force_x *= 2.0;
 	  intent.force_y *= 2.0;
